#!/usr/bin/python

import numpy
import random
import math
import matplotlib.pyplot as plt
from matplotlib.ticker import ScalarFormatter
import pylab

# neg feedback path
#  Vinp = Vout*R1/(R1+R2) = Vout*a

#  I4 = (Vinp-Vref)/R4
#  I3 = (Vout - Vinp) / R3
#  IL = Vinp / RL
#  IL = I3-I4
  
#  (Vout - Vinp) / R3 - (Vinp-Vref)/R4 = Vinp / RL
#  (Vo - Vo*a) / R3 - (Vo*a-Vref)/R4 = Vo*a / RL
#  R4 RL  (Vo - Vo*a)  - R3 RL (Vo*a-Vref) = Vo*a R3 R4
#  Vo ( R4 RL - a R4 RL ) - Vo * (R3 RL a - a R3 R4 )  = R3 RL Vref
#  Vo = R3 RL Vref
#      / ( R4 RL - a R4 RL - R3 RL a + a R3 R4 )

# calculate load-current
def howland_current(R1,R2,R3,R4,RL,Vref):
	Vo = float(    R3 *RL* Vref )
	a = float(R1)/float(R1+R2)
	Vo = Vo / float( R4* RL - a *R4* RL - R3* RL* a + a *R3* R4 )
	Vin = a*Vo
	return Vin / RL

# calculate output resistance
def howland_rout(R1,R2,R3,R4):
	Vref = 5
	RL_list = range(1,600) # range of load resistances
	I_list=[]
	V_list=[]
	for RL in RL_list:
		I = howland_current(R1,R2,R3,R4,RL,Vref)
		I_list.append(I)
		V_list.append(I*RL)
	return rout_from_list(I_list,V_list)

# from the list of voltages/currents, calculate output resistance
def rout_from_list(I_list,V_list):
	I_delta = I_list[-1] - I_list[1]
	V_delta = V_list[-1] - V_list[1]
	Rout = 0
	#if I_delta != 0:
	Rout = V_delta / I_delta
	return Rout

rout_list=[]
for n in range(10000):
	# 10k resistors with +/-0.05% tolerance matching
	R0 = 10e3
	mu = 0
	sigma = (0.05/100)*R0
	R1 = R0 + random.gauss(mu, sigma)
	R2 = R0 + random.gauss(mu, sigma)
	R3 = R0 + random.gauss(mu, sigma)
	R4 = R0 + random.gauss(mu, sigma)
	rout = howland_rout(R1,R2,R3,R4)
	if rout != 0:
		rout_list.append(abs(rout))

print len(rout_list)
print max(rout_list)
print min(rout_list)
print numpy.mean(rout_list)

histbins = [5e5*b for b in range(200)]
n, bins, patches = pylab.hist(rout_list,bins=histbins,normed=True)
pylab.setp(patches, 'facecolor', 'g')
ax = pylab.gca()
ax.set_xscale('log')
pylab.xlabel('Output Impedance (Ohm)')
pylab.title('Howland current-source.\n +/- 0.05% matched resistors. \n AW2013OCT19')
pylab.xlim([5e5,1e8])
pylab.show()

